/*
 * @(#)chopglue.java	1.00 31st Dec 1999
 * 
 * Modification Log:
 *
 */

import java.io.*;

/**
 * This is a file splitting and joining utility. 
 * Ultra light and platform independent.
 * <br>
 * To split you can specify the maximum size of each chunk or the number of
 * chunks you want. ChopGlue will create a set of files with names like
 * <filename>.0, <filename>.1 ..., where <filename> represents the original file.
 * It will also create a file called chop.control which stores vital information
 * required to glue the file back.
 * <br>
 * To recreate the file simply run chopglue with 'glue' as argument. It will look
 * for the chop.control file and the file chunks in the current directory and will
 * recreate the file.
 * 
 * <p><code>
 * Usage: <br>
 *        chopglue chop <file name> <-nNumber_of_chunks | -sChunk_size(bytes)><br>
 *        chopglue glue [control file name]<br>
 * </code></p>
 *
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 31st Dec, 1999
 */

public final class chopglue
{
	static String	sFileName, sControlFile;
	static int		iNumChunks;
	static long		lSizeChunks;
	static long		lFileSize;
	static File		file;
	static String	sERRSIZE = "Invalid size or number.";

	public static void main(String[] args)
	{
		if (args.length > 0) 
		{
			String		[]newArgs = new String[args.length -1];
			int			iIndex;
			for (iIndex=1; iIndex < args.length; iIndex++)
			{
				newArgs[iIndex-1] = args[iIndex];
			}

			try
			{
				if (args[0].equalsIgnoreCase("chop") && getChopParams(newArgs) && valChopParams())
				{
					writeCtrl();
					chopFile();
				}
				else if(args[0].equalsIgnoreCase("glue") && getGlueParams(newArgs) && valGlueParams())
				{
					readCtrl();
					glueFile();
				}
			}
			catch (IOException ioe)
			{
				System.out.println("\nError: " + ioe);
			}
		}
		else
		{
			System.out.println("Usage:");
			chopUsage();
			glueUsage();
		}
	}
	
	// ----------------------------------------------------------//
	//                   Glue implementation                     //
	// ----------------------------------------------------------//
	private static boolean readCtrl() throws IOException
	{
		DataInputStream dis = new DataInputStream(new FileInputStream("chop.control"));
		sFileName = dis.readUTF();
		iNumChunks = dis.readInt();
		lSizeChunks = dis.readLong();
		lFileSize = dis.readLong();
		dis.close();
		return true;
	}
	
	private static boolean glueFile() throws IOException
	{
		int						iIndex;
		BufferedInputStream		fis;
		BufferedOutputStream	fos = new BufferedOutputStream(new FileOutputStream(sFileName));
		byte					[]aData = new byte[4096];
		String					sChunkName;
		long					lReadTillNow;
		int						iReadNow, iToRead;

		System.out.println("Glued file: " + sFileName);
		System.out.println("File size: " + lFileSize + " bytes");
		System.out.println("Number of chunks: " + iNumChunks);
		System.out.println("Size of each chunk: " + lSizeChunks);
		for (iIndex=0; iIndex<iNumChunks; iIndex++)
		{
			sChunkName = sFileName + "." + (iIndex+1);
			fis = new BufferedInputStream(new FileInputStream(sChunkName));
			
			System.out.print(".");			
			for(lReadTillNow=0; lReadTillNow < lSizeChunks; lReadTillNow += iReadNow)
			{
				iToRead = ((lSizeChunks-lReadTillNow) > aData.length) ? aData.length : (int)(lSizeChunks-lReadTillNow);
				iReadNow = fis.read(aData, 0, iToRead);
				if (-1 == iReadNow) break;
				if (0 != iReadNow) fos.write(aData, 0, iReadNow);
			}
			fis.close();
		}
		fos.close();
		System.out.println("\nDone.");
		return true;
	}
	
	private static boolean valGlueParams()
	{
		file = new File(sControlFile);
		if(!file.exists() || !file.canRead())
		{
			System.out.println("Error accessing control file " + sControlFile);
			return false;
		}
		return true;
	}
	
	private static boolean getGlueParams(String[] args)
	{
		if(args.length < 2)
		{
			System.out.println("Control file: " + (sControlFile = ((1==args.length)?args[0]:"chop.control")));
			return true;
		}
		System.out.println("Incorrect command line.\nOnly one parameter expected.");
		glueUsage();
		return false;
	}
	
	private static void glueUsage()
	{
		System.out.println("chopglue glue [control file name]");
	}

	// ----------------------------------------------------------//
	//                   Chop implementation                     //
	// ----------------------------------------------------------//
	
	private static boolean writeCtrl() throws IOException
	{
		DataOutputStream dos = new DataOutputStream(new FileOutputStream("chop.control"));
		dos.writeUTF(sFileName);
		dos.writeInt(iNumChunks);
		dos.writeLong(lSizeChunks);
		dos.writeLong(lFileSize);
		dos.close();
		return true;
	}
	
	private static boolean chopFile() throws IOException
	{
		int						iIndex;
		BufferedInputStream		fis = new BufferedInputStream(new FileInputStream(file));
		BufferedOutputStream	fos;
		byte					[]aData = new byte[4096];
		String					sChunkName;
		long					lReadTillNow;
		int						iReadNow, iToRead;
		
		for (iIndex=0; iIndex<iNumChunks; iIndex++)
		{
			sChunkName = sFileName + "." + (iIndex+1);
			fos = new BufferedOutputStream(new FileOutputStream(sChunkName));
			
			System.out.print(".");
			for(lReadTillNow=0; lReadTillNow < lSizeChunks; lReadTillNow += iReadNow)
			{
				iToRead = ((lSizeChunks-lReadTillNow) > aData.length) ? aData.length : (int)(lSizeChunks-lReadTillNow);
				iReadNow = fis.read(aData, 0, iToRead);
				if (-1 == iReadNow) break;
				if (0 != iReadNow) fos.write(aData, 0, iReadNow);
			}
			fos.close();
		}
		fis.close();
		System.out.println("\nDone.");
		return true;
	}
	
	private static boolean valChopParams()
	{
		file = new File(sFileName);
		if(!file.exists() || !file.canRead())
		{
			System.out.println("Error accessing file " + sFileName);
			return false;
		}
		lFileSize = file.length();
		System.out.println("File size: " + lFileSize + " bytes");
		if(0 >= lFileSize)
		{
			System.out.println("No need to chop!");
			return false;
		}
		if(0 == iNumChunks)
		{
			iNumChunks = (int)(lFileSize/lSizeChunks);
			if(iNumChunks*lSizeChunks < lFileSize) iNumChunks++;
			System.out.println("Number of chunks: " + iNumChunks);
		}
		else
		{
			lSizeChunks = lFileSize/iNumChunks;
			if(iNumChunks*lSizeChunks < lFileSize) lSizeChunks++;
			System.out.println("Chunk size: " + lSizeChunks);
		}
		return true;
	}
	
	private static boolean getChopParams(String[] args)
	{
		if (args.length == 2)
		{
			System.out.println("File: " + (sFileName = args[0]));
			if(args[1].length() < 3)
			{
				System.out.println(sERRSIZE);
				chopUsage();
				return false;
			}
			try
			{
				if(args[1].startsWith("-n") || args[1].startsWith("-N"))
				{
					iNumChunks = Integer.parseInt(args[1].substring(2));
					lSizeChunks = 0;
					System.out.println("Number of chunks: " + iNumChunks);
				}
				else if(args[1].startsWith("-s") || args[1].startsWith("-S"))
				{
					lSizeChunks = Long.parseLong(args[1].substring(2));
					iNumChunks = 0;
					System.out.println("Size of each chunk: " + lSizeChunks);
				}
				if((0 >= lSizeChunks) && (1 >= iNumChunks))
				{
					System.out.println(sERRSIZE);
					chopUsage();
					return false;
				}
				return true;
			}
			catch(NumberFormatException ne)
			{
				System.out.println(sERRSIZE);
				chopUsage();
				return false;
			}
		}
		chopUsage();
		return false;
	}
	
	private static void chopUsage()
	{
		System.out.println("chopglue chop <file name> <-nNumber_of_chunks | -sChunk_size(bytes)>");
	}
}
